#!/usr/bin/env python3
# encoding: utf-8
#
# Copyright (C) 2020 Timm Fitschen <t.fitschen@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
#
"""
Accept a return request.
"""
# @review Timm Fitschen 2022-03-16
from __future__ import absolute_import

import caosdb as db
from caosadvancedtools.serverside.helper import get_timestamp, print_success

from .box_loan import (BOX, CONTENT, F_LOAN, RETURN_ACCEPTED, RETURNLOCATION,
                       S_RETURN_ACCEPTED, S_RETURN_REQUESTED, get_borrower_names, get_loan, main,
                       set_location_of_borrowed_items, set_property, update_loan_state)


def _accept_return_request(loan, location_updates=True, content_updates=True):
    """Update a loan Record and add the `accepted` Property."""
    # This changes the state from "return_requested" to "return_accepted".
    update_loan_state(loan, S_RETURN_ACCEPTED)

    if location_updates:
        items = set_location_of_borrowed_items(loan, RETURNLOCATION)
        if (content_updates and loan.get_property(CONTENT) is not None and
                loan.get_property(CONTENT).value):
            for item in items:
                if item.get_property(CONTENT) is not None:
                    item.get_property(CONTENT).value = loan.get_property(CONTENT).value
                else:
                    item.add_property(id=CONTENT.id,
                                      value=loan.get_property(CONTENT).value)

        items.update()
    loan.update()


def accept_return_request(data):
    """Accept a return request.

    I.e.add the `returnAccepted` Property to the `Loan` Record.
    """
    loan = get_loan(data[F_LOAN])

    config = db.get_config()
    location_updates = (not ("Misc" in config
                             and "entity_loan.no_location_updates" in config["Misc"]
                             and config["Misc"]["entity_loan.no_location_updates"]))
    content_updates = (not ("Misc" in config and "entity_loan.no_content_updates" in config["Misc"]
                            and config["Misc"]["entity_loan.no_content_updates"]))
    _accept_return_request(loan, location_updates=location_updates)
    fn, ln = get_borrower_names(loan)
    box_id = loan.get_property(BOX).value
    if not isinstance(box_id, list):
        box_id = [box_id]

    print_success('Thank you for accepting the return request by {fn} {ln}.<br>'
                  'If necessary, location and content of the box have been '
                  'updated.<br>'
                  '<a href="/Entity/{bid}" title="Reload this page.">Reload</a> '
                  'to view the new Location.<br>'
                  'See <a href="{loan}" title="Go to the accepted loan '
                  'request.">here</a>'.format(
                      fn=fn, ln=ln, loan=loan.id,
                      bid="&".join([str(el) for el in box_id])))
    return 0


if __name__ == "__main__":
    main(accept_return_request)
