#!/usr/bin/env python3
# encoding: utf-8
#
# Copyright (C) 2020 Timm Fitschen <t.fitschen@indiscale.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
#
"""
Manually return a box.
"""
from __future__ import absolute_import

import caosdb as db
from caosadvancedtools.serverside.helper import get_timestamp, print_success

from .box_loan import (BOX, BOX_BORROWED, BOX_RETURNED, CONTENT, F_LOAN, RETURNED, RETURNLOCATION,
                       S_RETURN_ACCEPTED, S_RETURNED, get_borrower_names, get_loan, main,
                       set_location_of_borrowed_items, set_property,
                       set_references_to_current_version, update_loan_state)


def _set_returned_box(loan):
    """Add a `Box (returned)` property to the loan.

    This stores the version of the box that was returned by a borrower.
    """
    references = set_references_to_current_version(loan.get_property(BOX_BORROWED).value)
    loan.add_property(id=BOX.id, name=BOX_RETURNED, value=references,
                      datatype=db.LIST(BOX.name) if isinstance(references, list) else BOX.name)


def set_return_location(loan):
    """Set the location of the box to the return location of the loan.  """
    config = db.get_config()
    if (not ("Misc" in config and "entity_loan.no_location_updates" in config["Misc"]
             and config["Misc"]["entity_loan.no_location_updates"])):
        items = set_location_of_borrowed_items(loan, RETURNLOCATION)
        items.update()


def set_content(loan):
    """Set the content to the box to the one given in the loan.  """
    config = db.get_config()
    if ("Misc" in config
            and "entity_loan.no_content_updates" in config["Misc"]
            and config["Misc"]["entity_loan.no_content_updates"]):
        return
    items = loan.get_property(BOX_BORROWED).value
    if not isinstance(items, list):
        items = [items]
    for item in items:
        box = db.Record(id=item).retrieve()
        set_property(box, CONTENT, loan.get_property(CONTENT).value)
        box.update()


def _manual_return(data):
    """Update a loan Record and add the `returned` Property."""
    loan = get_loan(data[F_LOAN])

    # This changes the state from "return_accepted" to "returned".
    update_loan_state(loan, S_RETURNED)

    # updates the box location and content
    # *currently this is not wanted*
    # set_return_location(loan)
    # set_content(loan)

    # add returned box
    _set_returned_box(loan)

    db.Container().extend([
        loan
    ]).update()

    return loan


def manual_return(data):
    """Return a box from a borrower.

    I.e. update the `Loan` Record and add the `returned` Property.
    """
    loan = _manual_return(data)
    fn, ln = get_borrower_names(loan)
    loan = get_loan(data[F_LOAN])
    box_id = loan.get_property(BOX).value

    if not isinstance(box_id, list):
        box_id = [box_id]
    print_success('The box borrowed by {fn} {ln} '.format(fn=fn, ln=ln)
                  + 'has been returned and the Box Location has been updated.'
                  '<a href="/Entity/{bid}" title="Reload this page.">Reload</a> '
                  'to view the new Location.<br>'.format(bid="&".join([str(el) for el in box_id])))

    return 0


if __name__ == "__main__":
    main(manual_return)
